@@ -17,7 +17,9 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-#  $Id: dpkg-cross.pl,v 1.39 2006/04/18 07:14:35 yoush-guest Exp $
+#  $Id: dpkg-cross.pl,v 1.42 2006/10/15 11:17:32 yoush Exp $
+
+use File::Temp qw/tempfile tempdir/;
 
 # Determine if the system wide or user defined cross-compile configuration
 # have to be read.
@@ -38,7 +40,7 @@ foreach my $var_ ( @intern_vars ) {
 }
 
 # Version of dpkg-cross environment.
-$DPKGCROSSVERSION = "1.26";
+$DPKGCROSSVERSION = "1.29";
 
 # Convertion table for Debian GNU/Linux architecture name (``$arch'') to GNU
 # type.
@@ -87,6 +89,7 @@ $DPKGCROSSVERSION = "1.26";
 	'arm' => 'ELF 32-bit LSB .* ARM',
 	'armeb' => 'ELF 32-bit MSB .* ARM',
 	'powerpc' => 'ELF 32-bit MSB .* PowerPC',
+	'powerpc64' => 'ELF 64-bit MSB .* PowerPC',
 	'mips' => 'ELF 32-bit MSB .* MIPS',
 	'mipsel' => 'ELF 32-bit LSB .* MIPS',
 	'hppa' => 'ELF 32-bit MSB .* PA-RISC',
@@ -109,6 +112,8 @@ $DPKGCROSSVERSION = "1.26";
 	"arm" => [ "arm-linux-gnu-", "arm-linux-" ],
 	"armeb" => [ "armeb-linux-gnu-", "armeb-linux-" ],
 	"powerpc" => [ "powerpc-linux-gnu-", "powerpc-linux-", "ppc-linux-gnu-", "ppc-linux-" ],
+	"powerpc64" => [ "powerpc64-linux-gnu-", "powerpc64-linux-", "ppc64-linux-gnu-", "ppc64-linux-",
+			 "powerpc-linux-gnu-", "powerpc-linux-", "ppc-linux-gnu-", "ppc-linux-" ],
 	"mips" => [ "mips-linux-gnu-", "mips-linux-", "mips64-linux-gnu-", "mips64-linux-" ],
 	"mipsel" => [ "mipsel-linux-gnu-", "mipsel-linux-", "mips64el-linux-gnu-", "mips64el-linux-" ],
 	"hppa" => [ "hppa-linux-gnu-", "hppa-linux-" ],
@@ -351,7 +356,10 @@ sub detect_arch {
 	my $file_ = shift (@_);
 
 	my $string_ = `LC_ALL=C file -L "$file_" 2>/dev/null`;
-	for my $arch_ (keys(%archdetecttable)) {
+	my $arch_ = $ENV{DPKGCROSSARCH} || $ENV{ARCH};
+	return $arch_ if $arch_;
+
+	for $arch_ (keys(%archdetecttable)) {
 		my $re_ = $archdetecttable{$arch_};
 		if ($string_ =~ /$re_/) {
 			if ($arch_ eq "AR") {
@@ -361,11 +369,11 @@ sub detect_arch {
 				my $obj_ = `ar t "$file_" 2>/dev/null | head -n1`;
 				chomp($obj_);	# remove newline
 				if ($obj_) {
-					my $tmp_ = `tempfile`;
-					chomp($tmp_);	# remove newline
+					my ($fh_, $tmp_) = tempfile(DIR => $ENV{'TMPDIR'} || '/tmp');
 					if (system("ar p \"$file_\" \"$obj_\" > $tmp_ 2>/dev/null") == 0) {
 						$arch_ = detect_arch($tmp_);
 					}
+					close $fh_;
 					unlink($tmp_);
 				}
 			}
@@ -624,17 +632,17 @@ sub convert_ld_library_path () {
 # $1: Directory basename (random suffix will be added)
 # return: Full directory pathname, undef if failed
 sub create_tmpdir ($) {
-	my ($pd, $d, $b, $try);
-	$b = $_[0];
-	$pd = $ENV{'TMPDIR'};
-	$pd = '/tmp' if ((!$pd) || (! -d $pd));
-	$try = 0;
-	while ($try < 100) {
-		$d = $pd . '/' . $b . '.' . sprintf("%08x", int(rand(0xFFFFFFFF)));
-		return $d if mkdir($d, 0700);
-		$try++;
-	}
-	return undef;
+	my $name = shift;
+	my $pd = $ENV{'TMPDIR'} && -d $ENV{'TMPDIR'}
+		? $ENV{'TMPDIR'}
+		: '/tmp';
+	return undef unless -d $pd;
+	my $dir;
+
+	eval { $dir = tempdir("$name.XXXXXXXX", DIR => $pd) };
+	print("$@"), return undef if $@;
+
+	return $dir;
 }
 
 1;
